#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 2005/2006 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005/2006 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import py.test

from ll import xpit


def test_plain():
	assert xpit.convert("foo") == "foo"
	assert xpit.convert("foo<?>bar") == "foo<?bar"


def test_expr():
	assert xpit.convert("<?= a+b?>", globals=dict(a=17, b=23)) == "40"


def test_if():
	assert xpit.convert("<?if a==17?>gurk<?else?>hurz<?endif?>", globals=dict(a=17, b=23)) == "gurk"
	assert xpit.convert("<?if a>0?>foo<?elif a==0?>bar<?else?>baz<?endif?>", globals=dict(a=17)) == "foo"


def test_nestedif():
	code = """
	<?if a==16?>
		a==16
	<?elif a==17?>
		a==17
		<?if b==17?>
			b==17
		<?else?>
			b!=17
		<?endif?>
	<?else?>
		a != (16, 17)
	<?endif?>
	"""
	assert "".join(xpit.convert(code, globals=dict(a=17, b=23)).split()) == "a==17b!=17"



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import re, datetime, StringIO

import py.test

from ll import ul4c


def check(result, source, **variables):
	# Check with template compiled from source
	t1 = ul4c.compile(source)
	s1 = unicode(t1)
	assert result == t1.renders(**variables)

	# Check with template loaded again via the string interface
	t2 = ul4c.loads(t1.dumps())
	s2 = unicode(t2)
	assert result == t2.renders(**variables)

	# Check with template loaded again via the stream interface
	stream = StringIO.StringIO()
	t1.dump(stream)
	stream.seek(0)
	t3 = ul4c.load(stream)
	s3 = unicode(t3)
	assert result == t3.renders(**variables)
	assert s1 == s2 == s3


def checkle(result, source, **variables):
	# Check with template compiled from source
	t1 = ul4c.compile(source)
	s1 = unicode(t1)
	assert result <= t1.renders(**variables)

	# Check with template loaded again via the string interface
	t2 = ul4c.loads(t1.dumps())
	s2 = unicode(t2)
	assert result <= t2.renders(**variables)

	# Check with template loaded again via the stream interface
	stream = StringIO.StringIO()
	t1.dump(stream)
	stream.seek(0)
	t3 = ul4c.load(stream)
	s3 = unicode(t3)
	assert result <= t3.renders(**variables)
	assert s1 == s2 == s3


def checkcompileerror(msg, source):
	try:
		ul4c.compile(source)
	except Exception, exc:
		assert re.search(msg, str(exc)) is not None
	else:
		py.test.fail("Didn't raise exception")


def checkrunerror(msg, source, **variables):
	# Check with template compiled from source
	t1 = ul4c.compile(source)
	s1 = unicode(t1)
	try:
		t1.renders(**variables)
	except Exception, exc:
		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
	else:
		py.test.fail("Didn't raise exception")

	# Check with template loaded again via the string interface
	t2 = ul4c.loads(t1.dumps())
	s2 = unicode(t2)
	try:
		t2.renders(**variables)
	except Exception, exc:
		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
	else:
		py.test.fail("Didn't raise exception")

	# Check with template loaded again via the stream interface
	stream = StringIO.StringIO()
	t1.dump(stream)
	stream.seek(0)
	t3 = ul4c.load(stream)
	s3 = unicode(t3)
	try:
		t3.renders(**variables)
	except Exception, exc:
		assert re.search(msg, "%s.%s: %s" % (exc.__class__.__module__, exc.__class__.__name__, exc)) is not None
	else:
		py.test.fail("Didn't raise exception")
	assert s1 == s2 == s3


def test_text():
	check(u'gurk', u'gurk')
	check(u'g\xfcrk', u'g\xfcrk')


def test_none():
	check('', u'<?print None?>')

	check('no', u'<?if None?>yes<?else?>no<?end if?>')


def test_false():
	check('False', u'<?print False?>')

	check('no', u'<?if False?>yes<?else?>no<?end if?>')


def test_true():
	check('True', u'<?print True?>')

	check('yes', u'<?if True?>yes<?else?>no<?end if?>')


def test_int():
	check('0', u'<?print 0?>')
	check('42', u'<?print 42?>')
	check('-42', u'<?print -42?>')
	check('255', u'<?print 0xff?>')
	check('255', u'<?print 0Xff?>')
	check('-255', u'<?print -0xff?>')
	check('-255', u'<?print -0Xff?>')
	check('63', u'<?print 0o77?>')
	check('63', u'<?print 0O77?>')
	check('-63', u'<?print -0o77?>')
	check('-63', u'<?print -0O77?>')
	check('7', u'<?print 0b111?>')
	check('7', u'<?print 0B111?>')
	check('-7', u'<?print -0b111?>')
	check('-7', u'<?print -0B111?>')

	check('no', u'<?if 0?>yes<?else?>no<?end if?>')
	check('yes', u'<?if 1?>yes<?else?>no<?end if?>')
	check('yes', u'<?if -1?>yes<?else?>no<?end if?>')


def test_float():
	check('0.0', u'<?print 0.?>')
	check('42.0', u'<?print 42.?>')
	check('-42.0', u'<?print -42.?>')
	check('1e+42', u'<?print 1E42?>')
	check('1e+42', u'<?print 1e42?>')
	check('-1e+42', u'<?print -1E42?>')
	check('-1e+42', u'<?print -1e42?>')

	check('no', u'<?if 0.?>yes<?else?>no<?end if?>')
	check('yes', u'<?if 1.?>yes<?else?>no<?end if?>')
	check('yes', u'<?if -1.?>yes<?else?>no<?end if?>')


def test_string():
	check('foo', u'<?print "foo"?>')
	check('\n', u'<?print "\\n"?>')
	check('\r', u'<?print "\\r"?>')
	check('\t', u'<?print "\\t"?>')
	check('\f', u'<?print "\\f"?>')
	check('\b', u'<?print "\\b"?>')
	check('\a', u'<?print "\\a"?>')
	check('\x1b', u'<?print "\\e"?>')
	check('"', u'<?print "\\""?>')
	check("'", u'<?print "\\\'"?>')
	check(u'\u20ac', u'<?print "\u20ac"?>')
	check(u'\xff', u'<?print "\\xff"?>')
	check(u'\u20ac', u'''<?print "\\u20ac"?>''')
	checkcompileerror("Unterminated string", u'<?print "?>')
	check("\\xxx", u'<?print "\\xxx"?>')
	check("a\nb", u'<?print "a\nb"?>')

	check('no', u'<?if ""?>yes<?else?>no<?end if?>')
	check('yes', u'<?if "foo"?>yes<?else?>no<?end if?>')


def test_date():
	check('2000-02-29T00:00:00', u'<?print 2000-02-29T.isoformat()?>')
	check('2000-02-29T12:34:00', u'<?print 2000-02-29T12:34.isoformat()?>')
	check('2000-02-29T12:34:56', u'<?print 2000-02-29T12:34:56.isoformat()?>')
	check('2000-02-29T12:34:56.987654', u'<?print 2000-02-29T12:34:56.987654.isoformat()?>')
	check('yes', u'<?if 2000-02-29T12:34:56.987654?>yes<?else?>no<?end if?>')


def test_list():
	check('', u'<?for item in []?><?print item?>;<?end for?>')
	check('1;', u'<?for item in [1]?><?print item?>;<?end for?>')
	check('1;', u'<?for item in [1,]?><?print item?>;<?end for?>')
	check('1;2;', u'<?for item in [1, 2]?><?print item?>;<?end for?>')
	check('1;2;', u'<?for item in [1, 2,]?><?print item?>;<?end for?>')
	check('no', u'<?if []?>yes<?else?>no<?end if?>')
	check('yes', u'<?if [1]?>yes<?else?>no<?end if?>')


def test_dict():
	check('', u'<?for (key, value) in {}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('1:2\n', u'<?for (key, value) in {1:2}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('1:2\n', u'<?for (key, value) in {1:2,}.items()?><?print key?>:<?print value?>\n<?end for?>')
	# With duplicate keys, later ones simply overwrite earlier ones
	check('1:3\n', u'<?for (key, value) in {1:2, 1: 3}.items()?><?print key?>:<?print value?>\n<?end for?>')
	# Test **
	check('1:2\n', u'<?for (key, value) in {**{1:2}}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('1:4\n', u'<?for (key, value) in {1:1, **{1:2}, 1:3, **{1:4}}.items()?><?print key?>:<?print value?>\n<?end for?>')
	check('no', u'<?if {}?>yes<?else?>no<?end if?>')
	check('yes', u'<?if {1:2}?>yes<?else?>no<?end if?>')


def test_code_storevar():
	check('42', u'<?code x = 42?><?print x?>')
	check('xyzzy', u'<?code x = "xyzzy"?><?print x?>')


def test_code_addvar():
	check('40', u'<?code x = 17?><?code x += 23?><?print x?>')
	check('xyzzy', u'<?code x = "xyz"?><?code x += "zy"?><?print x?>')


def test_code_subvar():
	check('-6', u'<?code x = 17?><?code x -= 23?><?print x?>')


def test_code_mulvar():
	check('391', u'<?code x = 17?><?code x *= 23?><?print x?>')
	check(17*'xyzzy', u'<?code x = 17?><?code x *= "xyzzy"?><?print x?>')
	check(17*'xyzzy', u'<?code x = "xyzzy"?><?code x *= 17?><?print x?>')


def test_code_floordivvar():
	check('2', u'<?code x = 5?><?code x //= 2?><?print x?>')
	check('-3', u'<?code x = -5?><?code x //= 2?><?print x?>')


def test_code_truedivvar():
	check('2.5', u'<?code x = 5?><?code x /= 2?><?print x?>')
	check('-2.5', u'<?code x = -5?><?code x /= 2?><?print x?>')


def test_code_modvar():
	check('4', u'<?code x = 1729?><?code x %= 23?><?print x?>')


def test_code_delvar():
	checkrunerror('KeyError', u'<?code x = 1729?><?code del x?><?print x?>')


def test_for_string():
	check('', u'<?for c in data?>(<?print c?>)<?end for?>', data="")
	check('(g)(u)(r)(k)', u'<?for c in data?>(<?print c?>)<?end for?>', data="gurk")


def test_for_list():
	check('', u'<?for c in data?>(<?print c?>)<?end for?>', data="")
	check('(g)(u)(r)(k)', u'<?for c in data?>(<?print c?>)<?end for?>', data=["g", "u", "r", "k"])


def test_for_dict():
	check('', u'<?for c in data?>(<?print c?>)<?end for?>', data={})
	check('(a)(b)(c)', u'<?for c in sorted(data)?>(<?print c?>)<?end for?>', data=dict(a=1, b=2, c=3))


def test_for_nested():
	check('[(1)(2)][(3)(4)]', u'<?for list in data?>[<?for n in list?>(<?print n?>)<?end for?>]<?end for?>', data=[[1, 2], [3, 4]])


def test_for_unpacking():
	data = [
		("spam", "eggs", 17),
		("gurk", "hurz", 23),
		("hinz", "kunz", 42)
	]
	check('(spam)(gurk)(hinz)', u'<?for (a,) in data?>(<?print a?>)<?end for?>', data=data)
	check('(spam,eggs)(gurk,hurz)(hinz,kunz)', u'<?for (a, b) in data?>(<?print a?>,<?print b?>)<?end for?>', data=data)
	check('(spam,eggs,17)(gurk,hurz,23)(hinz,kunz,42)', u'<?for (a, b, c) in data?>(<?print a?>,<?print b?>,<?print c?>)<?end for?>', data=data)


def test_break():
	check('1, 2, ', u'<?for i in [1,2,3]?><?print i?>, <?if i==2?><?break?><?end if?><?end for?>')


def test_break_nested():
	check('1, 1, 2, 1, 2, 3, ', u'<?for i in [1,2,3,4]?><?for j in [1,2,3,4]?><?print j?>, <?if j>=i?><?break?><?end if?><?end for?><?if i>=3?><?break?><?end if?><?end for?>')


def test_continue():
	check('1, 3, ', u'<?for i in [1,2,3]?><?if i==2?><?continue?><?end if?><?print i?>, <?end for?>')


def test_continue_nested():
	check('1, 3, \n1, 3, \n', u'<?for i in [1,2,3]?><?if i==2?><?continue?><?end if?><?for j in [1,2,3]?><?if j==2?><?continue?><?end if?><?print j?>, <?end for?>\n<?end for?>')


def test_if():
	check('42', u'<?if data?><?print data?><?end if?>', data=42)


def test_else():
	check('42', u'<?if data?><?print data?><?else?>no<?end if?>', data=42)
	check('no', u'<?if data?><?print data?><?else?>no<?end if?>', data=0)


def test_block_errors():
	checkcompileerror("in u?.<.for x in data.>..*block unclosed", u'<?for x in data?>')
	checkcompileerror("endif doesn't match any if", u'<?for x in data?><?end if?>')
	checkcompileerror("not in any block", u'<?end?>')
	checkcompileerror("not in any block", u'<?end for?>')
	checkcompileerror("not in any block", u'<?end if?>')
	checkcompileerror("else doesn't match any if", u'<?else?>')
	checkcompileerror("in u?.<.if data.>..*block unclosed", u'<?if data?>')
	checkcompileerror("in u?.<.if data.>..*block unclosed", u'<?if data?><?else?>')
	checkcompileerror("duplicate else", u'<?if data?><?else?><?else?>')
	checkcompileerror("else already seen in elif", u'<?if data?><?else?><?elif data?>')
	checkcompileerror("else already seen in elif", u'<?if data?><?elif data?><?elif data?><?else?><?elif data?>')


def test_empty():
	checkcompileerror("expression required", u'<?print?>')
	checkcompileerror("expression required", u'<?if?>')
	checkcompileerror("expression required", u'<<?if x?><?elif?><?end if?>')
	checkcompileerror("loop expression required", u'<?for?>')
	checkcompileerror("statement required", u'<?code?>')
	checkcompileerror("render statement required", u'<?render?>')


def test_add():
	check('42', u'<?print 21+21?>')
	check('42', u'<?code x=21?><?code y=21?><?print x+y?>')
	check('foobar', u'<?code x="foo"?><?code y="bar"?><?print x+y?>')
	check('(f)(o)(o)(b)(a)(r)', u'<?for i in data.foo+data.bar?>(<?print i?>)<?end for?>', data=dict(foo="foo", bar="bar"))


def test_sub():
	check('0', u'<?print 21-21?>')
	check('0', u'<?code x=21?><?code y=21?><?print x-y?>')



def test_mul():
	check(str(17*23), u'<?print 17*23?>')
	check(str(17*23), u'<?code x=17?><?code y=23?><?print x*y?>')
	check(17*"foo", u'<?print 17*"foo"?>')
	check(17*"foo", u'<?code x=17?><?code y="foo"?><?print x*y?>')
	check("foo"*17, u'<?code x="foo"?><?code y=17?><?print x*y?>')
	check("foo"*17, u'<?print "foo"*17?>')
	check("(foo)(bar)(foo)(bar)(foo)(bar)", u'<?for i in 3*data?>(<?print i?>)<?end for?>', data=["foo", "bar"])


def test_truediv():
	check("0.5", u'<?print 1/2?>')
	check("0.5", u'<?code x=1?><?code y=2?><?print x/y?>')


def test_floordiv():
	check("0", u'<?print 1//2?>')
	check("0", u'<?code x=1?><?code y=2?><?print x//y?>')


def test_mod():
	check(str(42%17), u'<?print 42%17?>')
	check(str(42%17), u'<?code x=42?><?code y=17?><?print x%y?>')


def test_eq():
	check("False", u'<?print 17==23?>')
	check("True", u'<?print 17==17?>')
	check("False", u'<?print x==23?>', x=17)
	check("True", u'<?print x==23?>', x=23)


def test_ne():
	check("True", u'<?print 17!=23?>')
	check("False", u'<?print 17!=17?>')
	check("True", u'<?print x!=23?>', x=17)
	check("False", u'<?print x!=23?>', x=23)


def test_lt():
	check("True", u'<?print 1<2?>')
	check("False", u'<?print 2<2?>')
	check("False", u'<?print 3<2?>')
	check("True", u'<?print x<2?>', x=1)
	check("False", u'<?print x<2?>', x=2)
	check("False", u'<?print x<2?>', x=3)


def test_le():
	check("True", u'<?print 1<=2?>')
	check("True", u'<?print 2<=2?>')
	check("False", u'<?print 3<=2?>')
	check("True", u'<?print x<=2?>', x=1)
	check("True", u'<?print x<=2?>', x=2)
	check("False", u'<?print x<=2?>', x=3)


def test_gt():
	check("False", u'<?print 1>2?>')
	check("False", u'<?print 2>2?>')
	check("True", u'<?print 3>2?>')
	check("False", u'<?print x>2?>', x=1)
	check("False", u'<?print x>2?>', x=2)
	check("True", u'<?print x>2?>', x=3)


def test_and():
	check("False", u'<?print x and y?>', x=False, y=False)
	check("False", u'<?print x and y?>', x=False, y=True)
	check("0", u'<?print x and y?>', x=0, y=True)


def test_or():
	check("False", u'<?print x or y?>', x=False, y=False)
	check("True", u'<?print x or y?>', x=False, y=True)
	check("42", u'<?print x or y?>', x=42, y=True)


def test_not():
	check("True", u'<?print not x?>', x=False)
	check("False", u'<?print not x?>', x=42)


def test_ge():
	check("False", u'<?print 1>=2?>')
	check("True", u'<?print 2>=2?>')
	check("True", u'<?print 3>=2?>')
	check("False", u'<?print x>=2?>', x=1)
	check("True", u'<?print x>=2?>', x=2)
	check("True", u'<?print x>=2?>', x=3)


def test_getitem():
	check("u", u"<?print 'gurk'[1]?>")
	check("u", u"<?print x[1]?>", x="gurk")
	check("u", u"<?print 'gurk'[-3]?>")
	check("u", u"<?print x[-3]?>", x="gurk")
	checkcompileerror("IndexError", u"<?print 'gurk'[4]?>")
	checkrunerror("IndexError", u"<?print x[4]?>", x="gurk")
	checkcompileerror("IndexError", u"<?print 'gurk'[-5]?>")
	checkrunerror("IndexError", u"<?print x[-5]?>", x="gurk")


def test_getslice12():
	check("ur", u"<?print 'gurk'[1:3]?>")
	check("ur", u"<?print x[1:3]?>", x="gurk")
	check("ur", u"<?print 'gurk'[-3:-1]?>")
	check("ur", u"<?print x[-3:-1]?>", x="gurk")
	check("", u"<?print 'gurk'[4:10]?>")
	check("", u"<?print x[4:10]?>", x="gurk")
	check("", u"<?print 'gurk'[-10:-5]?>")
	check("", u"<?print x[-10:-5]?>", x="gurk")


def test_getslice1():
	check("urk", u"<?print 'gurk'[1:]?>")
	check("urk", u"<?print x[1:]?>", x="gurk")
	check("urk", u"<?print 'gurk'[-3:]?>")
	check("urk", u"<?print x[-3:]?>", x="gurk")
	check("", u"<?print 'gurk'[4:]?>")
	check("", u"<?print x[4:]?>", x="gurk")
	check("gurk", u"<?print 'gurk'[-10:]?>")
	check("gurk", u"<?print x[-10:]?>", x="gurk")


def test_getslice2():
	check("gur", u"<?print 'gurk'[:3]?>")
	check("gur", u"<?print x[:3]?>", x="gurk")
	check("gur", u"<?print 'gurk'[:-1]?>")
	check("gur", u"<?print x[:-1]?>", x="gurk")
	check("gurk", u"<?print 'gurk'[:10]?>")
	check("gurk", u"<?print x[:10]?>", x="gurk")
	check("", u"<?print 'gurk'[:-5]?>")
	check("", u"<?print x[:-5]?>", x="gurk")


def test_nested():
	sc = u"4"
	sv = u"x"
	n = 4
	for i in xrange(8): # when using 10 compiling the variable will run out of registers
		sc = u"(%s)+(%s)" % (sc, sc)
		sv = u"(%s)+(%s)" % (sv, sv)
		n = n+n
	check(str(n), u'<?print %s?>' % sc)
	check(str(n), u'<?code x=4?><?print %s?>' % sv)


def test_precedence():
	check("14", u'<?print 2+3*4?>')
	check("20", u'<?print (2+3)*4?>')
	check("10", u'<?print -2+-3*-4?>')
	check("14", u'<?print --2+--3*--4?>')
	check("14", u'<?print (-(-2))+(-((-3)*-(-4)))?>')
	check("42", u'<?print 2*data.value?>', data=dict(value=21))
	check("42", u'<?print data.value[0]?>', data=dict(value=[42]))
	check("42", u'<?print data[0].value?>', data=[dict(value=42)])
	check("42", u'<?print data[0][0][0]?>', data=[[[42]]])
	check("42", u'<?print data.value.value[0]?>', data=dict(value=dict(value=[42])))
	check("42", u'<?print data.value.value[0].value.value[0]?>', data=dict(value=dict(value=[dict(value=dict(value=[42]))])))


def test_bracket():
	sc = u"4"
	sv = u"x"
	for i in xrange(10):
		sc = u"(%s)" % sc
		sv = u"(%s)" % sv

	check("4", u'<?print %s?>' % sc)
	check("4", u'<?code x=4?><?print %s?>' % sv)


def test_function_now():
	checkrunerror("function u?'now' unknown", u"<?print now(1)?>")
	checkrunerror("function u?'now' unknown", u"<?print now(1, 2)?>")
	now = unicode(datetime.datetime.now())
	checkle(now, u"<?print now()?>")


def test_function_vars():
	checkrunerror("function u?'vars' unknown", "<?print vars(1)?>")
	checkrunerror("function u?'vars' unknown", "<?print vars(1, 2)?>")
	check("yes", u"<?if 'spam' in vars()?>yes<?else?>no<?end if?>", spam="eggs")


def test_function_xmlescape():
	checkrunerror("function u?'xmlescape' unknown", u"<?print xmlescape()?>")
	checkrunerror("function u?'xmlescape' unknown", u"<?print xmlescape(1, 2)?>")
	check("&lt;&gt;&amp;&#39;&quot;gurk", u"<?print xmlescape(data)?>", data='<>&\'"gurk')


def test_function_csvescape():
	checkrunerror("function u?'csvescape' unknown", u"<?print csvescape()?>")
	checkrunerror("function u?'csvescape' unknown", u"<?print csvescape(1, 2)?>")
	check("", u"<?print csvescape(data)?>", data=None)
	check("False", u"<?print csvescape(data)?>", data=False)
	check("True", u"<?print csvescape(data)?>", data=True)
	check("42", u"<?print csvescape(data)?>", data=42)
	# no check for float
	check("abc", u"<?print csvescape(data)?>", data="abc")
	check('"a,b,c"', u"<?print csvescape(data)?>", data="a,b,c")
	check('"a""b""c"', u"<?print csvescape(data)?>", data='a"b"c')
	check('"a\nb\nc"', u"<?print csvescape(data)?>", data="a\nb\nc")


def test_function_str():
	checkrunerror("function u?'str' unknown", u"<?print str()?>")
	checkrunerror("function u?'str' unknown", u"<?print str(1, 2)?>")
	check("", u"<?print str(data)?>", data=None)
	check("True", u"<?print str(data)?>", data=True)
	check("False", u"<?print str(data)?>", data=False)
	check("42", u"<?print str(data)?>", data=42)
	check("4.2", u"<?print str(data)?>", data=4.2)
	check("foo", u"<?print str(data)?>", data="foo")


def test_function_int():
	checkrunerror("function u?'int' unknown", u"<?print int()?>")
	checkrunerror("function u?'int' unknown", u"<?print int(1, 2)?>")
	checkrunerror("int\\(\\) argument must be a string or a number, not 'NoneType'", u"<?print int(data)?>", data=None)
	check("1", u"<?print int(data)?>", data=True)
	check("0", u"<?print int(data)?>", data=False)
	check("42", u"<?print int(data)?>", data=42)
	check("4", u"<?print int(data)?>", data=4.2)
	check("42", u"<?print int(data)?>", data="42")
	checkrunerror("invalid literal for int\\(\\) with base 10: 'foo'", u"<?print int(data)?>", data="foo")


def test_function_len():
	checkrunerror("function u?'len' unknown", u"<?print len()?>")
	checkrunerror("function u?'len' unknown", u"<?print len(1, 2)?>")
	checkrunerror("object of type 'NoneType' has no len", u"<?print len(data)?>", data=None)
	checkrunerror("object of type 'bool' has no len", u"<?print len(data)?>", data=True)
	checkrunerror("object of type 'bool' has no len", u"<?print len(data)?>", data=False)
	checkrunerror("object of type 'int' has no len", u"<?print len(data)?>", data=42)
	checkrunerror("object of type 'float' has no len", u"<?print len(data)?>", data=4.2)
	check("42", u"<?print len(data)?>", data=42*"?")
	check("42", u"<?print len(data)?>", data=42*[None])
	check("42", u"<?print len(data)?>", data=dict.fromkeys(xrange(42)))


def test_function_enumerate():
	checkrunerror("function u?'enumerate' unknown", u"<?print enumerate()?>")
	checkrunerror("function u?'enumerate' unknown", u"<?print enumerate(1, 2)?>")
	code = u"<?for (i, value) in enumerate(data)?><?print i?>:<?print value?>\n<?end for?>"
	checkrunerror("'NoneType' object is not iterable", code, data=None)
	checkrunerror("'bool' object is not iterable", code, data=True)
	checkrunerror("'bool' object is not iterable", code, data=False)
	checkrunerror("'int' object is not iterable", code, data=42)
	checkrunerror("'float' object is not iterable", code, data=4.2)
	check("0:f\n1:o\n2:o\n", code, data="foo")
	check("0:foo\n1:bar\n", code, data=["foo", "bar"])
	check("0:foo\n", code, data=dict(foo=True))


def test_function_isnone():
	checkrunerror("function u?'isnone' unknown", u"<?print isnone()?>")
	checkrunerror("function u?'isnone' unknown", u"<?print isnone(1, 2)?>")
	code = u"<?print isnone(data)?>"
	check("True", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_isbool():
	checkrunerror("function u?'isbool' unknown", u"<?print isbool()?>")
	checkrunerror("function u?'isbool' unknown", u"<?print isbool(1, 2)?>")
	code = u"<?print isbool(data)?>"
	check("False", code, data=None)
	check("True", code, data=True)
	check("True", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_isint():
	checkrunerror("function u?'isint' unknown", u"<?print isint()?>")
	checkrunerror("function u?'isint' unknown", u"<?print isint(1, 2)?>")
	code = u"<?print isint(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("True", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_isfloat():
	checkrunerror("function u?'isfloat' unknown", u"<?print isfloat()?>")
	checkrunerror("function u?'isfloat' unknown", u"<?print isfloat(1, 2)?>")
	code = u"<?print isfloat(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("True", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_isstr():
	checkrunerror("function u?'isstr' unknown", u"<?print isstr()?>")
	checkrunerror("function u?'isstr' unknown", u"<?print isstr(1, 2)?>")
	code = u"<?print isstr(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("True", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_isdate():
	checkrunerror("function u?'isdate' unknown", u"<?print isdate()?>")
	checkrunerror("function u?'isdate' unknown", u"<?print isdate(1, 2)?>")
	code = u"<?print isdate(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("True", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_islist():
	checkrunerror("function u?'islist' unknown", u"<?print islist()?>")
	checkrunerror("function u?'islist' unknown", u"<?print islist(1, 2)?>")
	code = u"<?print islist(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("True", code, data=())
	check("True", code, data=[])
	check("False", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_isdict():
	checkrunerror("function u?'isdict' unknown", u"<?print isdict()?>")
	checkrunerror("function u?'isdict' unknown", u"<?print isdict(1, 2)?>")
	code = u"<?print isdict(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("True", code, data={})
	check("False", code, data=ul4c.compile(u""))


def test_function_istemplate():
	checkrunerror("function u?'istemplate' unknown", u"<?print istemplate()?>")
	checkrunerror("function u?'istemplate' unknown", u"<?print istemplate(1, 2)?>")
	code = u"<?print istemplate(data)?>"
	check("False", code, data=None)
	check("False", code, data=True)
	check("False", code, data=False)
	check("False", code, data=42)
	check("False", code, data=4.2)
	check("False", code, data="foo")
	check("False", code, data=datetime.datetime.now())
	check("False", code, data=())
	check("False", code, data=[])
	check("False", code, data={})
	check("True", code, data=ul4c.compile(u""))


def test_function_get():
	checkrunerror("function u?'get' unknown", u"<?print get()?>")
	check("", u"<?print get('x')?>")
	check("42", u"<?print get('x')?>", x=42)
	check("17", u"<?print get('x', 17)?>")
	check("42", u"<?print get('x', 17)?>", x=42)


def test_function_repr():
	checkrunerror("function u?'repr' unknown", u"<?print repr()?>")
	checkrunerror("function u?'repr' unknown", u"<?print repr(1, 2)?>")
	code = u"<?print repr(data)?>"
	check("None", code, data=None)
	check("True", code, data=True)
	check("False", code, data=False)
	check("42", code, data=42)
	# no test for float
	check("'foo'", code, data="foo")
	# no test for tuples, lists and dicts


def test_function_chr():
	checkrunerror("function u?'chr' unknown", u"<?print chr()?>")
	checkrunerror("function u?'chr' unknown", u"<?print chr(1, 2)?>")
	code = u"<?print chr(data)?>"
	check("\x00", code, data=0)
	check("a", code, data=ord("a"))
	check(u"\u20ac", code, data=0x20ac)


def test_function_ord():
	checkrunerror("function u?'ord' unknown", u"<?print ord()?>")
	checkrunerror("function u?'ord' unknown", u"<?print ord(1, 2)?>")
	code = u"<?print ord(data)?>"
	check("0", code, data="\x00")
	check(str(ord("a")), code, data="a")
	check(str(0x20ac), code, data=u"\u20ac")


def test_function_hex():
	checkrunerror("function u?'hex' unknown", u"<?print hex()?>")
	checkrunerror("function u?'hex' unknown", u"<?print hex(1, 2)?>")
	code = u"<?print hex(data)?>"
	check("0x0", code, data=0)
	check("0xff", code, data=0xff)
	check("0xffff", code, data=0xffff)
	check("-0xffff", code, data=-0xffff)


def test_function_oct():
	checkrunerror("function u?'oct' unknown", u"<?print oct()?>")
	checkrunerror("function u?'oct' unknown", u"<?print oct(1, 2)?>")
	code = u"<?print oct(data)?>"
	check("0o0", code, data=0)
	check("0o77", code, data=077)
	check("0o7777", code, data=07777)
	check("-0o7777", code, data=-07777)


def test_function_bin():
	checkrunerror("function u?'bin' unknown", u"<?print bin()?>")
	checkrunerror("function u?'bin' unknown", u"<?print bin(1, 2)?>")
	code = u"<?print bin(data)?>"
	check("0b0", code, data=0)
	check("0b11", code, data=3)
	check("-0b1111", code, data=-15)


def test_function_sorted():
	checkrunerror("function u?'sorted' unknown", u"<?print sorted()?>")
	checkrunerror("function u?'sorted' unknown", u"<?print sorted(1, 2)?>")
	code = u"<?for i in sorted(data)?><?print i?><?end for?>"
	check("gkru", code, data="gurk")
	check("24679", code, data="92746")
	check("012", code, data={0: "zero", 1: "one", 2: "two"})


def test_function_range():
	checkrunerror("function u?'sorted' unknown", u"<?print sorted()?>")
	code = u"<?for i in range(data)?><?print i?><?end for?>"
	check("", code, data=-10)
	check("", code, data=0)
	check("0", code, data=1)
	check("01234", code, data=5)
	code = u"<?for i in range(data[0], data[1])?><?print i?><?end for?>"
	check("", code, data=[0, -10])
	check("", code, data=[0, 0])
	check("01234", code, data=[0, 5])
	check("-5-4-3-2-101234", code, data=[-5, 5])
	code = u"<?for i in range(data[0], data[1], data[2])?><?print i?><?end for?>"
	check("", code, data=[0, -10, 1])
	check("", code, data=[0, 0, 1])
	check("02468", code, data=[0, 10, 2])
	check("", code, data=[0, 10, -2])
	check("108642", code, data=[10, 0, -2])
	check("", code, data=[10, 0, 2])


def test_function_zip():
	checkrunerror("function u?'zip' unknown", u"<?print zip()?>")
	checkrunerror("function u?'zip' unknown", u"<?print zip(1)?>")
	code = u"<?for (ix, iy) in zip(x, y)?><?print ix?>-<?print iy?>;<?end for?>"
	check("", code, x=[], y=[])
	check("1-3;2-4;", code, x=[1, 2], y=[3, 4])
	check("1-4;2-5;", code, x=[1, 2, 3], y=[4, 5])
	code = u"<?for (ix, iy, iz) in zip(x, y, z)?><?print ix?>-<?print iy?>+<?print iz?>;<?end for?>"
	check("", code, x=[], y=[], z=[])
	check("1-3+5;2-4+6;", code, x=[1, 2], y=[3, 4], z=[5, 6])
	check("1-4+6;", code, x=[1, 2, 3], y=[4, 5], z=[6])


def test_function_type():
	checkrunerror("function u?'type' unknown", u"<?print type()?>")
	checkrunerror("function u?'type' unknown", u"<?print type(1, 2)?>")
	code = u"<?print type(x)?>"
	check("none", code, x=None)
	check("bool", code, x=False)
	check("bool", code, x=True)
	check("int", code, x=42)
	check("int", code, x=42L)
	check("float", code, x=4.2)
	check("str", code, x="foo")
	check("str", code, x=u"foo")
	check("date", code, x=datetime.datetime.now())
	check("date", code, x=datetime.date.today())
	check("list", code, x=(1, 2))
	check("list", code, x=[1, 2])
	check("dict", code, x={1: 2})
	check("template", code, x=ul4c.compile(""))
	check("", code, x=1j)


def test_method_upper():
	check("GURK", u"<?print 'gurk'.upper()?>")


def test_method_lower():
	check("gurk", u"<?print 'GURK'.lower()?>")


def test_method_capitalize():
	check("Gurk", u"<?print 'gURK'.capitalize()?>")


def test_method_startswith():
	check("True", u"<?print 'gurkhurz'.startswith('gurk')?>")
	check("False", u"<?print 'gurkhurz'.startswith('hurz')?>")


def test_method_endswith():
	check("True", u"<?print 'gurkhurz'.endswith('hurz')?>")
	check("False", u"<?print 'gurkhurz'.endswith('gurk')?>")


def test_method_strip():
	check("gurk", r"<?print ' \t\r\ngurk \t\r\n'.strip()?>")
	check("gurk", r"<?print 'xyzzygurkxyzzy'.strip('xyz')?>")


def test_method_lstrip():
	check("gurk \t\r\n", ur"<?print ' \t\r\ngurk \t\r\n'.lstrip()?>")
	check("gurkxyzzy", ur"<?print 'xyzzygurkxyzzy'.lstrip('xyz')?>")


def test_method_rstrip():
	check(" \t\r\ngurk", ur"<?print ' \t\r\ngurk \t\r\n'.rstrip()?>")
	check("xyzzygurk", ur"<?print 'xyzzygurkxyzzy'.rstrip('xyz')?>")


def test_method_split():
	check("(g)(u)(r)(k)", ur"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.split()?>(<?print item?>)<?end for?>")
	check("(g)(u \t\r\nr \t\r\nk \t\r\n)", ur"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.split(None, 1)?>(<?print item?>)<?end for?>")
	check("()(g)(u)(r)(k)()", ur"<?for item in 'xxgxxuxxrxxkxx'.split('xx')?>(<?print item?>)<?end for?>")
	check("()(g)(uxxrxxkxx)", ur"<?for item in 'xxgxxuxxrxxkxx'.split('xx', 2)?>(<?print item?>)<?end for?>")


def test_method_rsplit():
	check("(g)(u)(r)(k)", ur"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.rsplit()?>(<?print item?>)<?end for?>")
	check("( \t\r\ng \t\r\nu \t\r\nr)(k)", ur"<?for item in ' \t\r\ng \t\r\nu \t\r\nr \t\r\nk \t\r\n'.rsplit(None, 1)?>(<?print item?>)<?end for?>")
	check("()(g)(u)(r)(k)()", ur"<?for item in 'xxgxxuxxrxxkxx'.rsplit('xx')?>(<?print item?>)<?end for?>")
	check("(xxgxxuxxr)(k)()", ur"<?for item in 'xxgxxuxxrxxkxx'.rsplit('xx', 2)?>(<?print item?>)<?end for?>")


def test_method_replace():
	check('goork', ur"<?print 'gurk'.replace('u', 'oo')?>")


def test_method_format():
	now = datetime.datetime.now()
	format = "%Y-%m-%d %H:%M:%S"
	check(now.strftime(format), ur"<?print data.format('%s')?>" % format, data=now)


def test_method_isoformat():
	now = datetime.datetime.now()
	check(now.isoformat(), ur"<?print data.isoformat()?>", data=now)


def test_method_get():
	check("42", u"<?print {}.get('foo', 42)?>")
	check("17", u"<?print {'foo': 17}.get('foo', 42)?>")
	check("", u"<?print {}.get('foo')?>")
	check("17", u"<?print {'foo': 17}.get('foo')?>")


def test_render():
	t = ul4c.compile(u'<?print prefix?><?print data?><?print suffix?>')
	check('(f)(o)(o)', u'<?for c in data?><?render t(data=c, prefix="(", suffix=")")?><?end for?>', t=t, data='foo')
	check('(f)(o)(o)', u'<?for c in data?><?render t(data=c, **{"prefix": "(", "suffix": ")"})?><?end for?>', t=t, data='foo')


def test_render_var():
	t = ul4c.compile(u'<?code x += 1?><?print x?>')
	check('42,43,42', u'<?print x?>,<?render t(x=x)?>,<?print x?>', t=t, x=42)


def test_parse():
	check('42', u'<?print data.Noner?>', data=dict(Noner=42))


def test_nested_exceptions():
	tmpl1 = ul4c.compile(u"<?print 2*x?>")
	tmpl2 = ul4c.compile(u"<?render tmpl1(x=x)?>")
	tmpl3 = ul4c.compile(u"<?render tmpl2(tmpl1=tmpl1, x=x)?>")

	checkrunerror(r"TypeError .*render tmpl3.*render tmpl2.*render tmpl1.*print 2.*unsupported operand type", u"<?render tmpl3(tmpl1=tmpl1, tmpl2=tmpl2, x=x)?>", tmpl1=tmpl1, tmpl2=tmpl2, tmpl3=tmpl3, x=None)


def test_note():
	check("foo", u"f<?note This is?>o<?note a comment?>o")



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 2005-2007 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005-2007 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


import py.test

from ll import misc


def test_pool():
	pool = misc.Pool(misc)
	assert pool.Pool is pool["Pool"] is misc.Pool



#! /usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 2000-2008 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2000-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
:mod:`ll.sisyphus` simplifies running Python stuff as cron jobs.

There will be no more than one sisyphus job of a certain name running at every
given time. When the job is already running and a second is started, the second
one will quit immediately if the first one hasn't exceeded its maximum allowed
lifetime yet. If it has exceeded the allowed lifetime the first job
will be killed and the second will start running.

In addition to that, logging features are provided.

To use this module, you must derive your own class from :class:`Job` and
implement the :meth:`execute` method.

The job announces its presence (and its process id) in a file that is stored in
the :dir:`~/run` directory. Logs will be created in the :dir:`~/log` directory
(This can be changes by deriving new subclasses and overwriting the appropriate
class attribute).

There are three log files:

	:file:`~/log/jobname_progress.log`
		The progress of one job execution is logged here. This log file will be
		truncated at the start of every execution, so you can be rather verbose.
		Use the :meth:`logProgress` method for writing to this log file.

	:file:`~/log/jobname_loop.log`
		One log line may be appended to the log after every job execution.
		:meth:`logLoop` method for this.

	:file:`~/log/jobname_error.log`
		Here exceptions happening during the execution of a job will be logged.
		This is done via the :meth:`logError` method with can be used for
		reporting other exception conditions apart from exceptions.

To execute a job, use the module level function :func:`execute`.

Example
-------

The following example illustrates the use of this module::

	#!/usr/bin/env python

	import os
	import urllib
	from ll import sisyphus

	class Fetch(sisyphus.Job):
		"savely fetches an HTML file and saves it to a local file."

		def __init__(self):
			sisyphus.Job.__init__(self, 180, name="Fetch")
			self.url = "http://www.python.org/"
			self.tmpname = "Fetch_Tmp_%d.html" % os.getpid()
			self.officialname = "Python.html"

		def execute(self):
			self.logProgress("fetching data from %r" % self.url)
			data = urllib.urlopen(self.url).read()
			datasize = len(data)
			self.logProgress("writing file %r (%d bytes)" % (self.tmpname, datasize))
			open(self.tmpname, "wb").write(data)
			self.logProgress("renaming file %r to %r" % (self.tmpname, self.officialname))
			os.rename(self.tmpname, self.officialname)
			self.logLoop("cached %r as %r (%d bytes)" % (self.url, self.officialname, datasize))

	if __name__=="__main__":
		sisyphus.execute(Fetch())
"""


import sys, os, traceback, errno, pprint, datetime

from ll import url


__docformat__ = "reStructuredText"


def _formattime(timestamp):
	"""
	Format :var:`timestamp` into a string.
	"""
	return timestamp.strftime("%d/%b/%Y %H:%M:%S")


def _formattimedelta(timedelta):
	"""
	Format :var:`timedelta` into a string.
	"""
	rest = timedelta.seconds
	(rest, secs) = divmod(rest, 60)
	(rest, mins) = divmod(rest, 60)
	rest += timedelta.days*24
	return "%d:%02d:%06.3f" % (rest, mins, secs+timedelta.microseconds/1000000.)


class LogFile:
	"""
	A log file. All lines written to the file will be prepended with a time stamp.
	"""
	def __init__(self, name, mode="a", buffering=True, encoding="iso-8859-1"):
		"""
		Create a new log file (which will be opened on the first write).
		Arguments are:

		:var:`name`
			The filename (either as a string or a :class:`ll.url.URL` instance).

		:var:`mode`
			The mode for opening the file (should be ``"w"`` or ``"a"``).

		:var:`buffering`
			The buffering for the file (``0`` is unbuffered, ``1`` is line
			buffered, any other integer specifies the buffersize).

		:var:`encoding`
			The encoding to use for the strings written to the file.
		"""
		self.starttime = datetime.datetime.now()
		if not isinstance(name, url.URL):
			name = url.File(name)
		self.name = name
		self.mode = mode
		self.buffering = buffering
		self.encoding = encoding
		self.file = None

	def __open(self):
		if self.file is None:
			self.file = open(self.name.local(), self.mode, self.buffering)

	def write(self, *texts):
		"""
		Write :var:`texts` to the log file.
		"""
		now = datetime.datetime.now()
		pid = os.getpid()
		prefix = "[pid=%d][%s]=[t+%s]" % (pid, _formattime(now), _formattimedelta(now-self.starttime))

		self.__open()
		for text in texts:
			if isinstance(text, str):
				pass
			elif isinstance(text, unicode):
				text = text.encode(self.encoding, "replace")
			elif isinstance(text, Exception):
				tb = "\n" + "".join(traceback.format_tb(sys.exc_info()[-1]))
				text = "%s%s: %s" % (tb, text.__class__.__name__, text)
			else:
				text = pprint.pformat(text)
			lines = text.splitlines()
			if lines and not len(lines[-1]):
				del lines[-1]
			for line in lines:
				self.file.write("%s %s\n" % (prefix, line))


class Job(object):
	"""
	A Job object executes a task once.
	
	The job announces its presence (and its process id) in a file that is stored
	in the :dir:`~/run` directory. Logs will be created in the :dir:`~/log`
	directory (This can be changes by deriving new subclasses).

	To use this class, derive your own class from it and overwrite the
	:meth:`execute` method.
	"""

	pidfilenametemplate = "~/run/%s.pid"
	loopfilenametemplate = "~/log/%s_loop.log"
	errorfilenametemplate = "~/log/%s_error.log"
	progressfilenametemplate = "~/log/%s_progress.log"

	def __init__(self, maxruntime=0, name=None, raiseerrors=False, printkills=False):
		"""
		Create a new job. Arguments are:

		:var:`maxruntime`: : integer
			The maximum allowed runtime in seconds for this job;

		:var:`name`: : string or :const:`None`
			The name to be used for the log files. If :const:`None``, the name of
			the class will be used;

		:var:`raiseerrors`: : bool
			should exceptions that occur during the excution of the job be raised
			(which results in a output to the terminal, or an email from the cron
			daemon);

		:var:`printkills`: : bool
			should the fact that a previous job was killed, be printed on stdout
			(resulting in a mail from the cron daemon)
		"""
		self.starttime = datetime.datetime.now()
		if not isinstance(maxruntime, datetime.timedelta):
			maxruntime = datetime.timedelta(seconds=maxruntime)
		self.maxruntime = maxruntime
		self.name = name or self.__class__.__name__
		self.raiseerrors = raiseerrors
		self.printkills = printkills
		self.pidfilewritten = False
		self.pidfilename = url.File(self.pidfilenametemplate % self.name)
		self.loopLogfile = LogFile(self.loopfilenametemplate % self.name)
		self.errorLogfile = LogFile(self.errorfilenametemplate % self.name)
		self.progressLogfile = LogFile(self.progressfilenametemplate % self.name, mode="w")

	def __writepid(self):
		"""
		Create the file containing the pid of the current process.
		"""
		if not self.pidfilewritten:
			file = self.pidfilename.openwrite()
			file.write(str(os.getpid()))
			file.close()
			self.pidfilewritten = True

	def __killpid(self):
		"""
		Delete the pid file.
		"""
		if self.pidfilewritten:
			self.pidfilename.remove()
			self.pidfilewritten = False

	def logLoop(self, *texts):
		"""
		Log the message texts to the loop and progress log. (The call to
		:meth:`logLoop` should be the last statement in the :meth:`execute`
		method.)
		"""
		self.loopLogfile.write(*texts)
		self.progressLogfile.write(*texts)

	def logProgress(self, *texts):
		"""
		Log the message texts to the progress log.
		"""
		self.progressLogfile.write(*texts)

	def logErrorOnly(self, *texts):
		"""
		Log the error to the error log. :var:`texts` may be strings or exception
		objects.
		"""
		self.errorLogfile.write(*texts)

	def logError(self, error):
		"""
		Log the error to the error log and the progress log. :var:`texts` may be
		strings or exception objects.
		"""
		self.progressLogfile.write(error)
		self.errorLogfile.write(error)

	def execute(self):
		"""
		Execute the job once. At the end of the job you should write one line to
		the loop log. Overwrite in subclasses.
		"""
		self.logLoop("done")

	def failed(self):
		"""
		Called when running the job generated an exception. Overwrite in
		subclasses, to e.g. rollback your database transactions.
		"""
		pass

	def handleexecution(self):
		"""
		Handle executing the job including handling of duplicate or hanging jobs.
		This is the method to be called from the outside world.
		"""
		try: # is there a pid file from a running job?
			file = open(self.pidfilename.local(), "r")
		except IOError, exc: # no pid file => the job has been finished without problems
			if exc[0] == errno.ENOENT: # file not found
				self.__writepid()
				self.logProgress("no previous job running; here we go")
			else:
				raise
		else:
			lastmodified = datetime.datetime.fromtimestamp(os.fstat(file.fileno()).st_mtime)
			pid = int(file.read())
			file.close()

			# Check if this process really exists, if not continue as if the pid file wasn't there
			try:
				os.kill(pid, 0)
			except OSError, exc:
				if exc[0] != errno.ESRCH:
					raise
				self.__writepid()
				msg = "ignoring bogus pid file %s (process with pid %d doesn't exist)" % (self.pidfilename, pid)
				self.logError(msg)
			else:
				if self.maxruntime and self.starttime-lastmodified > self.maxruntime: # the job is to old, so it probably hangs => kill it
					try:
						os.kill(pid, 9)
					except OSError, exc:
						if exc[0] != errno.ESRCH: # there was no process
							raise
					self.__writepid()
					msg = "killed previous job running with pid %d (ran %s seconds; %s allowed); here we go" % (pid, _formattimedelta(self.starttime-lastmodified), _formattimedelta(self.maxruntime))
					self.logError(msg)
					if self.printkills:
						print msg
				else:
					msg = "Job still running (for %s; %s allowed; started on %s) with pid %d (according to %s)" % (_formattimedelta(self.starttime-lastmodified), _formattimedelta(self.maxruntime), _formattime(lastmodified), pid, self.pidfilename)
					self.logErrorOnly(msg)
					return # Return without calling :meth:`execute`

		try:
			self.execute()
		except (Exception, KeyboardInterrupt), exc:
			self.logError(exc) # log the error
			self.logLoop("failed with %s(%s)" % (exc.__class__.__name__, exc)) # log the error to the loop log too, because the job probably didn't have a chance to do it.
			self.failed()
			if self.raiseerrors or isinstance(exc, KeyboardInterrupt): # Really exit
				self.__killpid()
				raise
		self.__killpid() # finished => remove the pid file


def execute(*jobs):
	"""
	Execute several jobs.

	Items in :var:`jobs` are job objects, that will be executed sequentially.
	"""
	for job in jobs:
		job.handleexecution()



# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
An XIST module that contains the global attributes from Zopes "Macro Expansion
Template Attribute Language".
"""


from ll.xist import xsc


__docformat__ = "reStructuredText"


xmlns = "http://xml.zope.org/namespaces/metal"


class Attrs(xsc.Attrs):
	class define_macro(xsc.TextAttr):
		xmlns = xmlns
		xmlname = "define-macro"

	class use_macro(xsc.TextAttr):
		xmlns = xmlns
		xmlname = "use-macro"

	class define_slot(xsc.TextAttr):
		xmlns = xmlns
		xmlname = "define-slot"

	class use_slot(xsc.TextAttr):
		xmlns = xmlns
		xmlname = "use-slot"



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


from __future__ import with_statement

import py.test

from ll.xist import xsc
from ll.xist.ns import html, php, chars, abbr


def test_basics():
	# empty pool
	r = xsc.Pool()
	py.test.raises(xsc.IllegalElementError, r.elementclass, "a", html)
	py.test.raises(xsc.IllegalElementError, r.elementclass_xml, "a", html)

	# register one element
	r = xsc.Pool(html.a)
	assert r.elementclass("a", html) is html.a
	assert r.elementclass_xml("a", html) is html.a
	py.test.raises(xsc.IllegalElementError, r.elementclass, "b", html)
	py.test.raises(xsc.IllegalElementError, r.elementclass_xml, "b", html)

	# register a module
	r = xsc.Pool(html)
	assert r.elementclass("a", html) is html.a
	assert r.elementclass_xml("a", html) is html.a
	assert r.elementclass("b", html) is html.b
	assert r.elementclass_xml("b", html) is html.b
	py.test.raises(xsc.IllegalElementError, r.elementclass, "c", html)
	py.test.raises(xsc.IllegalElementError, r.elementclass_xml, "c", html)

	# procinsts
	r = xsc.Pool(php.php)
	assert r.procinstclass("php") is php.php
	assert r.procinstclass_xml("php") is php.php
	assert r.procinst("php", "foo") == php.php("foo")
	assert r.procinst_xml("php", "foo") == php.php("foo")
	py.test.raises(xsc.IllegalProcInstError, r.procinstclass, "nophp")
	py.test.raises(xsc.IllegalProcInstError, r.procinstclass_xml, "nophp")

	# entities
	r = xsc.Pool(abbr)
	assert r.entityclass("xist") is abbr.xist
	assert r.entityclass_xml("xist") is abbr.xist
	assert r.entity("xist") == abbr.xist()
	assert r.entity_xml("xist") == abbr.xist()
	py.test.raises(xsc.IllegalEntityError, r.entityclass, "dontxist")
	py.test.raises(xsc.IllegalEntityError, r.entityclass_xml, "dontxist")

	# charrefs
	r = xsc.Pool(chars)
	assert r.charrefclass("ouml") is chars.ouml
	assert r.charrefclass_xml("ouml") is chars.ouml
	assert r.charrefclass(ord(u"ö")) is chars.ouml
	assert r.charrefclass_xml(ord(u"ö")) is chars.ouml
	assert r.charref("ouml") == chars.ouml()
	assert r.charref_xml("ouml") == chars.ouml()
	assert r.charref(ord(u"ö")) == chars.ouml()
	assert r.charref_xml(ord(u"ö")) == chars.ouml()
	py.test.raises(xsc.IllegalEntityError, r.charrefclass, "nothing")
	py.test.raises(xsc.IllegalEntityError, r.charrefclass_xml, "nothing")


def test_textcomment():
	r = xsc.Pool()
	assert r.text("foo") == xsc.Text("foo")
	assert r.comment("foo") == xsc.Comment("foo")


def test_defaultpool():
	r = xsc.threadlocalpool.pool
	assert r.elementclass("a", html) is html.a
	assert r.elementclass_xml("a", html) is html.a
	assert r.procinstclass("php") is php.php
	assert r.procinstclass_xml("php") is php.php
	assert r.entityclass("xist") is abbr.xist
	assert r.entityclass_xml("xist") is abbr.xist
	assert r.charrefclass("euro") is chars.euro
	assert r.charrefclass_xml("euro") is chars.euro
	assert r.charrefclass(ord(u"\N{EURO SIGN}")) is chars.euro
	assert r.charrefclass_xml(ord(u"\N{EURO SIGN}")) is chars.euro


def test_names():
	# Test classes where the Python and the XML name differ
	with xsc.Pool() as r:
		class element(xsc.Element):
			xmlname = "-element"
			xmlns = "nix"
		class procinst(xsc.ProcInst):
			xmlname = "-procinst"
		class entity(xsc.Entity):
			xmlname = "-entity"
		class charref(xsc.CharRef):
			xmlname = "-charref"
			codepoint = 42
		class Attrs(xsc.Attrs):
			class attr(xsc.TextAttr):
				xmlname = "-attr"
				xmlns = "nix"

	# elements
	assert r.elementclass("element", "nix") is element
	assert r.elementclass_xml("-element", "nix") is element
	assert r.element("element", "nix") == element()
	assert r.element_xml("-element", "nix") == element()
	py.test.raises(xsc.IllegalElementError, r.elementclass, "-element", "nix")
	py.test.raises(xsc.IllegalElementError, r.elementclass_xml, "element", "nix")
	# make sure that the default pool didn't pick up the new class
	py.test.raises(xsc.IllegalElementError, xsc.threadlocalpool.pool.elementclass, "element", "nix")
	py.test.raises(xsc.IllegalElementError, xsc.threadlocalpool.pool.elementclass_xml, "-element", "nix")

	# procinsts
	assert r.procinstclass("procinst") is procinst
	assert r.procinstclass_xml("-procinst") is procinst
	assert r.procinst("procinst", "spam") == procinst("spam")
	assert r.procinst_xml("-procinst", "spam") == procinst("spam")
	py.test.raises(xsc.IllegalProcInstError, r.procinstclass, "-procinst")
	py.test.raises(xsc.IllegalProcInstError, r.procinstclass_xml, "procinst")
	# make sure that the default pool didn't pick up the new class
	py.test.raises(xsc.IllegalProcInstError, xsc.threadlocalpool.pool.procinstclass, "procinst")
	py.test.raises(xsc.IllegalProcInstError, xsc.threadlocalpool.pool.procinstclass_xml, "-procinst")

	# entities
	assert r.entityclass("entity") is entity
	assert r.entityclass_xml("-entity") is entity
	assert r.entity("entity") == entity()
	assert r.entity_xml("-entity") == entity()
	py.test.raises(xsc.IllegalEntityError, r.entityclass, "-entity")
	py.test.raises(xsc.IllegalEntityError, r.entityclass_xml, "entity")
	# make sure that the default pool didn't pick up the new class
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.entityclass, "entity")
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.entityclass_xml, "-entity")
	# the charref is an entity too
	assert r.entityclass("charref") is charref
	assert r.entityclass_xml("-charref") is charref
	assert r.entity("charref") == charref()
	assert r.entity_xml("-charref") == charref()
	py.test.raises(xsc.IllegalEntityError, r.entityclass, "-charref")
	py.test.raises(xsc.IllegalEntityError, r.entityclass_xml, "charref")
	# make sure that the default pool didn't pick up the new class
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.entityclass, "charref")
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.entityclass_xml, "-charref")

	# charrefs
	assert r.charrefclass("charref") is charref
	assert r.charrefclass_xml("-charref") is charref
	assert r.charrefclass(42) is charref
	assert r.charrefclass_xml(42) is charref
	assert r.charref("charref") == charref()
	assert r.charref_xml("-charref") == charref()
	assert r.charref(42) == charref()
	assert r.charref_xml(42) == charref()
	py.test.raises(xsc.IllegalEntityError, r.charrefclass, "-charref")
	py.test.raises(xsc.IllegalEntityError, r.charrefclass_xml, "charref")
	# make sure that the default pool didn't pick up the new class
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.charrefclass, "charref")
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.charrefclass_xml, "-charref")
	# make sure that entity has not been register as a charref
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.charrefclass, "entity")
	py.test.raises(xsc.IllegalEntityError, xsc.threadlocalpool.pool.charrefclass_xml, "-entity")

	# attributes
	assert r.attrclass("attr", "nix") is Attrs.attr
	assert r.attrclass_xml("-attr", "nix") is Attrs.attr
	py.test.raises(xsc.IllegalAttrError, r.attrclass, "-attr", "nix")
	py.test.raises(xsc.IllegalAttrError, r.attrclass_xml, "attr", "nix")
	# make sure that the default pool didn't pick up the new class
	py.test.raises(xsc.IllegalAttrError, xsc.threadlocalpool.pool.attrclass, "attr", "nix")
	py.test.raises(xsc.IllegalAttrError, xsc.threadlocalpool.pool.attrclass_xml, "-attr", "nix")


def test_names2():
	with xsc.Pool() as r:
		class el_(xsc.Element):
			xmlname = "el"
		class en_(xsc.Entity):
			xmlname = "en"
		class pi_(xsc.ProcInst):
			xmlname = "pi"
		class cr_(xsc.CharRef):
			xmlname = "cr"
			codepoint = 0x4242

	# Test elements
	assert set(r.elements()) == set([el_])

	# Test entities
	assert set(r.entities()) == set([cr_, en_])

	# Test procinsts
	assert set(r.procinsts()) == set([pi_])

	# Test charrefs
	assert set(r.charrefs()) == set([cr_])


def test_stack():
	with xsc.Pool() as r1:
		class foo1(xsc.Element):
			xmlname = "foo"
			xmlns = "nix"
		with xsc.Pool() as r2:
			class foo2(xsc.Element):
				xmlname = "foo"
				xmlns = "nix"

	assert r1.elementclass_xml("foo", "nix") is foo1
	assert r2.elementclass_xml("foo", "nix") is foo2


def test_chain():
	with xsc.Pool() as p1:
		class foo1(xsc.Element):
			xmlname = "foo"
			xmlns = "nix"
		class baz(xsc.Element):
			xmlns = "nix"

	with xsc.Pool(p1) as p2:
		class foo2(xsc.Element):
			xmlname = "foo"
			xmlns = "nix"
		class bar(xsc.Element):
			xmlns = "nix"

	assert p1.elementclass_xml("foo", "nix") is foo1
	py.test.raises(xsc.IllegalElementError, p1.elementclass, "bar", "nix")
	assert p1.elementclass_xml("baz", "nix") is baz

	assert p2.elementclass_xml("foo", "nix") is foo2
	assert p2.elementclass_xml("bar", "nix") is bar
	assert p2.elementclass_xml("baz", "nix") is baz


def test_chain2():
	with xsc.Pool() as p1:
		class foo1(xsc.Element):
			xmlns = "nix"

	with xsc.Pool() as p2:
		class foo2(xsc.Element):
			xmlns = "nix"

	p = xsc.Pool(p1, p2)
	assert p.elementclass_xml("foo2", "nix") is foo2


def test_mixedattrnames():
	with xsc.Pool() as r:
		class Attrs(xsc.Attrs):
			class a(xsc.TextAttr):
				xmlns = "test"
				xmlname = "A"
			class A(xsc.TextAttr):
				xmlns = "test"
				xmlname = "a"

		class Test(xsc.Element):
			xmlns = "test"
			class Attrs(xsc.Element.Attrs):
				class a(xsc.TextAttr):
					xmlname = "A"
				class A(xsc.TextAttr):
					xmlname = "a"

	xmlns = "test"

	node = Test(
		{
			Attrs.a: "a2",
			Attrs.A: "A2",
		},
		a="a",
		A="A"
	)

	def check(name, value):
		assert unicode(node[name]) == value
		assert unicode(node.attrs[name]) == value
		if isinstance(name, basestring):
			assert unicode(getattr(node.attrs, name)) == value
		assert unicode(node.attrs.get(name)) == value
		if isinstance(name, basestring):
			assert unicode(node.attrs.get_xml(name.swapcase())) == value

	tests = [
		("a", "a"),
		("A", "A"),
		(Test.Attrs.a, "a"),
		(Test.Attrs.A, "A"),
		(Attrs.a, "a2"),
		(Attrs.A, "A2")
	]
	for (name, value) in tests:
		yield check, name, value

def test_xmlns():
	p = xsc.Pool(html)
	assert p.xmlns == html.xmlns



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


from __future__ import with_statement

import warnings, cStringIO

from xml.etree import cElementTree

import py.test

from xml import sax
from xml.parsers import expat

from ll import url
from ll.xist import xsc, parsers
from ll.xist.ns import xml, chars, html, ihtml, specials, ruby


oldfilters = None


class a(xsc.Element):
	xmlns = "http://www.example.com/foo"
	class Attrs(xsc.Element.Attrs):
		class id(xsc.IDAttr): pass
		class title(xsc.TextAttr): pass


class foo(xsc.Entity):
	def __unicode__(self):
		return u"FOO"


class bar(xsc.CharRef):
	codepoint = 0x42


def check_parseentities(source, result, **parseargs):
	node = parsers.parsestring("""<a title="%s">%s</a>""" % (source, source), **parseargs)
	node = node.walknode(xsc.FindType(xsc.Element))[0]
	assert unicode(node) == result
	assert unicode(node["title"]) == result


def check_parsestrictentities(source, result, parserfactory):
	prefixes = {None: (a.xmlns, chars)}
	check_parseentities(source, result, prefixes=prefixes, parser=parserfactory())

	for bad in ("&", "&#x", "&&", "&#x;", "&#fg;", "&#999999999;", "&#;", "&#y;", "&#x;", "&#xy;"):
		py.test.raises(expat.ExpatError, check_parseentities, bad, u"", prefixes=prefixes, parser=parserfactory())
	py.test.raises(xsc.IllegalEntityError, check_parseentities, "&baz;", u"", prefixes=prefixes, parser=parserfactory())


def test_parsingmethods():
	t = u"abc\U00012345\u3042xyz"
	s = u'<?xml version="1.0" encoding="utf-8"?><a title="%s">%s</a>' % (t, t)
	b = s.encode("utf-8")

	def check(node):
		node = node.walknode(a)[0]
		assert unicode(node) == t
		assert unicode(node["title"]) == t

	yield check, parsers.parsestring(b, pool=xsc.Pool(a))
	yield check, parsers.parsestring(s, pool=xsc.Pool(a)) # parsestring can parse unicode directly
	yield check, parsers.parseiter(b, pool=xsc.Pool(a)) # parse byte by byte
	yield check, parsers.parsestream(cStringIO.StringIO(b), bufsize=1, pool=xsc.Pool(a))
	yield check, parsers.parseetree(cElementTree.fromstring(b), prefixes={None: [a.xmlns]}, pool=xsc.Pool(a))


def test_parselocationsgmlop():
	# sgmlop doesn't provide any location info, so check only the URL
	node = parsers.parsestring("<z>gurk&amp;hurz&#42;hinz&#x666;hunz</z>", parser=parsers.SGMLOPParser())
	assert len(node) == 1
	assert len(node[0]) == 1
	assert str(node[0][0].startloc.url) == "STRING"
	assert node[0][0].startloc.line is None
	assert node[0][0].startloc.col is None


def test_parselocationexpat():
	# Check that expat gets the location info right
	node = parsers.parsestring("<z>gurk&amp;hurz&#42;hinz&#x666;hunz</z>", parser=parsers.ExpatParser())
	assert len(node) == 1
	assert len(node[0]) == 1
	assert str(node[0][0].startloc.url) == "STRING"
	assert node[0][0].startloc.line == 0
	assert node[0][0].startloc.col == 36 # expat reports the *end* of the text


class Test:
	def setup_method(self, method):
		global oldfilters
		oldfilters = warnings.filters[:]

	def teardown_method(self, method):
		warnings.filters = oldfilters

	def test_nsparse(self):
		# A prepopulated prefix mapping and xmlns attributes should work together
		xml = """
			<x:a>
				<x:a xmlns:x='http://www.w3.org/1999/xhtml'>
					<x:a xmlns:x='http://www.nttdocomo.co.jp/imode'>gurk</x:a>
				</x:a>
			</x:a>
		"""
		check = ihtml.a(
			html.a(
				ihtml.a(
					"gurk"
				)
			)
		)
		node = parsers.parsestring(xml, prefixes=dict(x=ihtml))
		node = node.walknode(xsc.FindType(xsc.Element))[0].compact() # get rid of the Frag and whitespace
		assert node == check

	def test_parseurls(self):
		# Check proper URL handling when parsing URLAttr or StyleAttr attributes
		node = parsers.parsestring('<a href="4.html" style="background-image: url(3.gif);"/>', base="root:1/2.html", prefixes={None: html})
		assert str(node[0]["style"]) == "background-image: url(root:1/3.gif)"
		assert node[0]["style"].urls() == [url.URL("root:1/3.gif")]
		assert str(node[0]["href"]) == "root:1/4.html"
		assert node[0]["href"].forInput(root="gurk/hurz.html") == url.URL("gurk/1/4.html")

	def test_parserequiredattrs(self):
		xmlns = "http://www.example.com/required"

		prefixes = {None: xmlns}

		# Parser should complain about required attributes that are missing
		with xsc.Pool():
			class Test(xsc.Element):
				xmlns = "http://www.example.com/required"
				class Attrs(xsc.Element.Attrs):
					class required(xsc.TextAttr):
						required = True

			node = parsers.parsestring('<Test required="foo"/>', prefixes=prefixes)
			assert str(node[0]["required"]) == "foo"
	
			warnings.filterwarnings("error", category=xsc.RequiredAttrMissingWarning)
			py.test.raises(xsc.RequiredAttrMissingWarning, parsers.parsestring, '<Test/>', prefixes=prefixes)

		py.test.raises(xsc.IllegalElementError, parsers.parsestring, '<Test required="foo"/>', prefixes=prefixes)

	def test_parsevalueattrs(self):
		xmlns = "http://www.example.com/required2"

		prefixes = {None: xmlns}

		# Parser should complain about attributes with illegal values, when a set of values is specified
		with xsc.Pool():
			class Test(xsc.Element):
				xmlns = "http://www.example.com/required2"
				class Attrs(xsc.Element.Attrs):
					class withvalues(xsc.TextAttr):
						values = ("foo", "bar")

			node = parsers.parsestring('<Test withvalues="bar"/>', prefixes=prefixes)
			assert str(node[0]["withvalues"]) == "bar"
	
			warnings.filterwarnings("error", category=xsc.IllegalAttrValueWarning)
			py.test.raises(xsc.IllegalAttrValueWarning, parsers.parsestring, '<Test withvalues="baz"/>', prefixes=prefixes)

	def test_parsestrictentities_expat(self):
		check_parsestrictentities(
			"a&amp;bc&#32;d&#x20;&#30000;;&lt;&gt;&quot;&apos;",
			u"""a&bc d %c;<>"'""" % 30000,
			parsers.ExpatParser
		)


def test_multipleparsecalls():
	def check(parser):
		b = parsers.Builder(parser=parser)
		for i in xrange(3):
			try:
				b.parsestring("<>gurk")
			except Exception:
				pass
			for j in xrange(3):
				assert b.parsestring("<a>gurk</a>").bytes() == "<a>gurk</a>"

	# A Parser instance should be able to parse multiple XML sources, even when some of the parse calls fail
	for parser in (parsers.SGMLOPParser, parsers.ExpatParser):
		yield check, parser()


def test_parseentities_sgmlop():
	def check(input, output):
		prefixes = {None: (a.xmlns, chars)}
		node = parsers.parsestring("""<a title="%s">%s</a>""" % (input, input), parser=parsers.SGMLOPParser(), prefixes=prefixes)
		node = node.walknode(a)[0]
		assert unicode(node) == output
		assert unicode(node.attrs.title) == output

	yield check, "a", "a"
	yield check, ";a;", ";a;"
	yield check, "&lt;", "<"
	yield check, "&lt;&gt;", "<>"
	yield check, "&gt;", ">"
	yield check, "&apos;", "'"
	yield check, "&quot;", '"'
	yield check, "&amp;", "&"
	yield check, "&amp;", "&"
	yield check, "a&amp;b", "a&b"
	yield check, "&foo;", "FOO"
	yield check, "&bar;", "\x42"
	yield check, "&#32;", " "
	yield check, "&#x20;", " "
	yield check, "&#x3042;", u"\u3042"


def test_parseattr_sgmlop():
	def check(input, output):
		prefixes = {None: (a.xmlns, chars)}
		node = parsers.parsestring(input, parser=parsers.SGMLOPParser(), prefixes=prefixes)
		node = node.walknode(a)[0]
		assert unicode(node.attrs.title) == output

	yield check, """<a title=x></a>""", "x"
	yield check, """<a title=x/>""", "x"
	yield check, """<a title=x id=42/>""", "x"
	yield check, """<a title="x" id=42/>""", "x"
	yield check, """<a title='x' id=42/>""", "x"
	yield check, """<a title='x"y' id=42/>""", 'x"y'
	yield check, """<a title="x'y" id=42/>""", "x'y"


def test_parsestringurl():
	# Base URLs should end up in the location info of the resulting XML tree
	node = parsers.parsestring("gurk", parser=parsers.SGMLOPParser())
	assert str(node[0].startloc.url) == "STRING"

	node = parsers.parsestring("gurk", base="root:gurk.xmlxsc", parser=parsers.SGMLOPParser())
	assert str(node[0].startloc.url) == "root:gurk.xmlxsc"


def test_xmlns():
	s = "<z xmlns=%r><rb xmlns=%r/><z/></z>" % (specials.xmlns, ruby.xmlns)
	e = parsers.parsestring(s)

	assert e[0].xmlns == specials.xmlns
	assert e[0][0].xmlns == ruby.xmlns

	s = "<a xmlns=%r><a xmlns=%r/></a>" % (html.xmlns, ihtml.xmlns)
	e = parsers.parsestring(s, pool=xsc.Pool(html, ihtml))
	assert isinstance(e[0], html.a)
	assert isinstance(e[0][0], ihtml.a)

	s = "<a><a xmlns=%r/></a>" % ihtml.xmlns
	py.test.raises(xsc.IllegalElementError, parsers.parsestring, s, prefixes={None: html}, pool=xsc.Pool(ihtml))
	e = parsers.parsestring(s, prefixes={None: html}, pool=xsc.Pool(html, ihtml))
	assert isinstance(e[0], html.a)
	assert isinstance(e[0][0], ihtml.a)

	s = "<z xmlns=%r/>" % specials.xmlns
	e = parsers.parsestring(s, pool=xsc.Pool(specials.z))
	assert isinstance(e[0], specials.z)
	py.test.raises(xsc.IllegalElementError, parsers.parsestring, s, pool=xsc.Pool())


def test_parseemptyattribute():
	e = parsers.parsestring("<a target=''/>", pool=xsc.Pool(html))
	assert "target" in e[0].attrs


def test_expat_xmldecl():
	e = parsers.parsestring("<?xml version='1.0' encoding='utf-8' standalone='yes'?><a/>", parser=parsers.ExpatParser())
	assert not isinstance(e[0], xml.XML)

	e = parsers.parsestring("<a/>", parser=parsers.ExpatParser(xmldecl=True))
	assert not isinstance(e[0], xml.XML)

	e = parsers.parsestring("<?xml version='1.0'?><a/>", parser=parsers.ExpatParser(xmldecl=True))
	assert isinstance(e[0], xml.XML)
	assert e[0].content == u'version="1.0"'

	e = parsers.parsestring("<?xml version='1.0' encoding='utf-8'?><a/>", parser=parsers.ExpatParser(xmldecl=True))
	assert isinstance(e[0], xml.XML)
	assert e[0].content == u'version="1.0" encoding="utf-8"'

	e = parsers.parsestring("<?xml version='1.0' encoding='utf-8' standalone='yes'?><a/>", parser=parsers.ExpatParser(xmldecl=True))
	assert isinstance(e[0], xml.XML)
	assert e[0].content == u'version="1.0" encoding="utf-8" standalone="yes"'


def test_expat_doctype():
	e = parsers.parsestring('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"><a/>', parser=parsers.ExpatParser())
	assert not isinstance(e[0], xsc.DocType)

	e = parsers.parsestring('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"><a/>', parser=parsers.ExpatParser(doctype=True))
	assert isinstance(e[0], xsc.DocType)
	assert e[0].content == html.DocTypeXHTML11().content

	e = parsers.parsestring('<!DOCTYPE html><a/>', parser=parsers.ExpatParser(doctype=True))
	assert isinstance(e[0], xsc.DocType)
	assert e[0].content == "html"

	e = parsers.parsestring('<!DOCTYPE html SYSTEM "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"><a/>', parser=parsers.ExpatParser(doctype=True))
	assert isinstance(e[0], xsc.DocType)
	assert e[0].content == u'html SYSTEM "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"'

	e = parsers.parsestring('<!DOCTYPE a [<!ELEMENT a EMPTY><!--gurk-->]><a/>', parser=parsers.ExpatParser(doctype=True))
	assert isinstance(e[0], xsc.DocType)
	assert e[0].content == u'a' # Internal subset gets dropped


def test_htmlparse_base():
	e = parsers.parsestring("<a href='gurk.gif'/>", base="hurz/index.html", tidy=True)
	e = e.walknode(html.a)[0]
	assert unicode(e.attrs.href) == "hurz/gurk.gif"



#! /usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 2000-2008 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2000-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


r"""
<p><mod>ll.ansistyle</mod> is a module that helps colorize terminal output
with &ansi; escape sequences.</p>

<p>Color values are integers between -1 and 511 (octal 0777).
The lowest 3 bits are used for the background colors 0-7. Bits 3-5 are used for
the foreground color. Bit 6 (0100) is used for bold (or other shades of the
color 0-7, depending on your terminal). Bit 7 (0200) is used for underlined text
and bit 8 (0400) is used for blinking text. The color value -1 means
<z>don't change the color</z>.</p>

<p>To add color switching escape sequences to normal output test, use a
<class>Colorizer</class> object like this:</p>
<tty>
<prompt>&gt;&gt;&gt; </prompt><input>from ll import ansistyle</input>
<prompt>&gt;&gt;&gt; </prompt><input>c = ansistyle.Colorizer()</input>
<prompt>&gt;&gt;&gt; </prompt><input>list(c.feed("spam", 0157, "eggs", None))</input>
['spam', '\x1b[1;35;47m', 'eggs', '\x1b[0m']
</tty>

<p>For more info see the description of the
<pyref class="Colorizer" method="feed"><meth>feed</meth></pyref>.</p>

<p>There is a second level &api; for <class>ansistyle.Colorizer</class>, which
can be used like this:</p>

<tty>
<prompt>&gt;&gt;&gt; </prompt><input>from ll import ansistyle</input>
<prompt>&gt;&gt;&gt; </prompt><input>list(ansistyle.Text("spam", 0157, "eggs").parts())</input>
['spam', '\x1b[1;35;47m', 'eggs', '\x1b[0m']
</tty>

<p>Furthermore you can print <class>ansistyle.Text</class> instances directly
to get colored output.</p>

<p>For more information see the documentation for
<pyref class="Text"><class>ansistyle.Text</class></pyref>.</p>
"""


from _ansistyle import switchcolor


__docformat__ = "xist"


class Colorizer(object):
	"""
	A <class>Colorizer</class> object manages the current color and style and will
	intersperse normal output text with &ansi; escape sequences for switching
	output color and style.
	"""

	def __init__(self, colored=True):
		"""
		Create a <class>Colorizer</class> instance. If <arg>colored</arg> is false,
		output will never contain any color/style switching escape sequences.
		"""
		self.colored = colored
		self._colors = [0070]
		self._activecolor = 0070

	def pushcolor(self, color):
		"""
		Push <arg>color</arg> onto the color stack
		"""
		self._colors.append(color)

	def popcolor(self):
		"""
		Pop a color from the color stack.
		"""
		return self._colors.pop(-1)

	def _switchcolor(self, color):
		if self.colored and color != -1:
			result = switchcolor(self._activecolor, color)
			self._activecolor = color
			return result
		return ""

	def feed(self, *strings):
		"""
		<p>This method is a generator and will yield all the strings in <arg>strings</arg>
		with interspersed color switching escape sequences. Items in <arg>strings</arg>
		can be the following:</p>
		<dl>
		<term>Strings (<class>str</class> and <class>unicode</class>)</term>
		<item>Strings will be output by <meth>feed</meth> in the appropriate spot.</item>
		<term>Numbers</term>
		<item>A number in the argument sequence will switch to that color value.</item>
		<term><lit>None</lit></term>
		<item>This will switch back to the default color (This is different from
		using the color number 0070, because 0070 will only switch colors if there
		is some output string afterwards).</item>
		<term>Sequences</term>
		<item>Those will be recursively fed to <meth>feed</meth> with the following
		added functionality: The color that was active before the start of the
		sequence will be restored after the end.</item>
		</dl>
		"""
		for string in strings:
			if isinstance(string, (int, long)):
				if string != -1:
					self._colors[-1] = string
			elif string is None:
				part = self._switchcolor(0070)
				if part:
					yield part
			elif isinstance(string, basestring):
				if string:
					part = self._switchcolor(self._colors[-1])
					if part:
						yield part
					yield string
			else:
				self.pushcolor(self._colors[-1]) # duplicate current color
				for part in self.feed(*string):
					yield part
				self.popcolor()


class Text(list):
	"""
	A colored string. A <class>Text</class> object is a sequence, the sequence
	items may either be strings or <class>Text</class> objects themselves.
	"""

	def __init__(self, *content):
		list.__init__(self, content)

	def append(self, *others):
		list.extend(self, others)

	def __call__(self, *others):
		list.extend(self, others)
		return self

	def insert(self, index, *others):
		list.__setslice__(self, index, index, list(others))

	def __repr__(self):
		return "%s.%s(%s)" % (self.__class__.__module__, self.__class__.__name__, list.__repr__(self)[1:-1])

	def parts(self, colored=True):
		"""
		This generator yields the strings that will make up the final colorized string.
		"""
		colorizer = Colorizer(colored)
		for part in colorizer.feed(self, None):
			yield part

	def string(self, colored=True):
		"""
		Return the resulting string (with escape sequences, if <arg>colored</arg> is true).
		"""
		return "".join(self.parts(colored))

	def __str__(self):
		"""
		Return the resulting string with &ansi; color escape sequences.
		"""
		return self.string(True)


class EscapedText(Text):
	"""
	An extension to the <class>Text</class> class. Special characters can be
	replaced by customized <class>Text</class> objects via the
	<meth>escapechar</meth> method.
	"""
	def __init__(self, *content):
		newcontent = []
		for text in content:
			if isinstance(text, basestring):
				wasstr = None
				for c in text:
					c = self.escapechar(c)
					isstr = isinstance(c, basestring)
					if wasstr and isstr:
						newcontent[-1] += c
					else:
						newcontent.append(c)
					wasstr = isstr
			else:
				newcontent.append(text)
		Text.__init__(self, *newcontent)

	def escapechar(self, char):
		"""
		Return a replacement <class>Text</class> object for the character
		<arg>char</arg> or <arg>char</arg> itself, if the character should be
		used as is. This method should be overwritten by subclasses.
		"""
		return char



#! /usr/bin/env python
# -*- coding: utf-8 -*-

## Copyright 2005-2008 by LivingLogic AG, Bayreuth/Germany.
## Copyright 2005-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
:mod:`ll.xpit` contains functions that make it possible to embed Python
expressions and conditionals in text. For example::

	from ll import xpit

	text = '''
	a = <?= a?>
	b = <?= b?>
	The sum is <?= a+b?>
	<?if a>0?>a is positive<?elif a==0?>a is 0<?else?>a is negative<?endif?>
	'''

	print xpit.convert(text, dict(a=23, b=42))

This will print::

	a = 23
	b = 42
	The sum is 65
	a is positive
"""


__docformat__ = "reStructuredText"


def tokenize(string):
	"""
	Tokenize the string :var:`string` and split it into processing instructions
	and text. :func:`tokenize` will generate tuples with the first item being
	the processing instruction target and the second being the PI data. "Text"
	content (i.e. anything other than PIs) will be returned as ``(None, data)``.
	A literal ``<?`` can be written as ``<?>`` and will be returned as text.
	"""
	pos = 0
	while True:
		pos1 = string.find("<?", pos)
		if pos1<0:
			part = string[pos:]
			if part:
				yield (None, part)
			return
		pos2 = string.find("?>", pos1)
		if pos2<0:
			part = string[pos:]
			if part:
				yield (None, part)
			return
		elif pos2 == pos1+1: # <?>
			yield (None, string[pos:pos1+2])
			pos = pos1+3
			continue
		part = string[pos:pos1]
		if part:
			yield (None, part)
		parts = string[pos1+2:pos2].split(None, 1)
		if len(parts) > 1:
			yield tuple(parts)
		else:
			yield (parts[0], parts[0][:0]) # empty string of correct type as data
		pos = pos2+2


class UnknownTargetError(ValueError):
	"""
	Exception that is raised when an unknown PI target (i.e. anything except
	``=``, ``if``, ``elif``, ``else``, ``endif``) is encountered.
	"""
	def __init__(self, target):
		self.target = target

	def __str__(self):
		return "Unknown PI target %s" % self.target


def convert(string, globals=None, locals=None):
	"""
	Convert :var:`string` using :var:`globals` and :var:`locals` as the global
	and local namespace.

	All processing instructions in :var:`string` with the target ``=`` (e.g.
	``<?=23+42?>``) will be evaluated with :var:`globals` as the global and
	:var:`locals` as the local namespace. Plain text will be passed through
	literally. Other allowed PI targets are ``if``, ``else``, ``elif`` and
	``endif``. These PIs implement conditional output. The PI content of ``if``
	and ``elif`` is evaluated as a Python expression. If it is true, everything
	after this PI (up to the next ``else``, ``endif`` etc.) will be included in
	the output. All these PIs will have :var:`globals` as the global and
	:var:`locals` as the local namespace.

	Processing instructions with other targets will raise an
	:exc:`UnknownTargetError` exception.
	"""
	v = []
	conds = [] # stack of (condition type, if-expression, else-expresion)
	for (action, data) in tokenize(string):
		if action is None:
			if all(ifcond for (type, ifcond, notelsecond) in conds):
				v.append(data)
		elif action == "if":
			cond = eval(data, globals, locals)
			conds.append(("if", cond, not cond))
		elif action == "elif":
			cond = eval(data, globals, locals)
			conds[-1] = ("elif", cond, conds[-1][2] and not cond)
		elif action == "else":
			conds[-1] = ("else", conds[-1][2], False)
		elif action == "endif":
			del conds[-1]
		elif action == "=":
			if all(ifcond for (type, ifcond, notelsecond) in conds):
				data = str(eval(data, globals, locals))
				v.append(data)
		elif action is not None:
			raise UnknownTargetError(action)
	return "".join(v)



# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


"""
Contains everthing related to options in XIST (apart for syntax highlighting
which can be found in presenters.py).
"""


import sys, os


__docformat__ = "reStructuredText"


def getenvstr(name, default):
	try:
		return os.environ[name]
	except:
		return default


def getenvint(name, default):
	try:
		return int(os.environ[name])
	except:
		return default


repransi = getenvint("LL_XIST_REPRANSI", 0)  # should ANSI escape sequences be used for dumping the DOM tree and which ones? (0=off, 1=dark background, 2=light background)
reprtab = getenvstr("LL_XIST_REPR_TAB", u"  ") # how to represent an indentation in the DOM tree?
reprencoding = sys.getdefaultencoding()

server = "localhost" # Host for server relative URLs



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license

import py.test

from ll.xist import xsc
from ll.xist.ns import html, xml

import xist_common as common


def test_walk_coverage():
	node = common.createfrag()
	class Filter(xsc.WalkFilter):
		def filternode(self, node):
			return (True, xsc.enterattrs, xsc.entercontent, True)

	# call only for code coverage
	for path in node.walk(Filter()):
		pass


def test_walk_result():
	def check(node, filter, result):
		filter = filter()
		def path2str(path):
			return ".".join("#" if isinstance(node, xsc.Text) else node.xmlname for node in path)

		assert map(path2str, node.walkpath(filter)) == result

	node = html.div(
		html.tr(
			html.th("gurk"),
			html.td("hurz"),
			id=html.b(42)
		),
		class_=html.i("hinz")
	)

	class filtertopdown(xsc.WalkFilter):
		def filternode(self, node):
			return (isinstance(node, xsc.Element), xsc.entercontent)

	class filterbottomup(xsc.WalkFilter):
		def filternode(self, node):
			return (xsc.entercontent, isinstance(node, xsc.Element))

	class filtertopdownattrs(xsc.WalkFilter):
		def filternode(self, node):
			return (isinstance(node, xsc.Element), xsc.enterattrs, xsc.entercontent)

	class filterbottomupattrs(xsc.WalkFilter):
		def filternode(self, node):
			return (xsc.enterattrs, xsc.entercontent, isinstance(node, xsc.Element))

	class filtertopdowntextonlyinattr(xsc.WalkFilter):
		def filterpath(self, path):
			for node in path:
				if isinstance(node, xsc.Attr):
					inattr = True
					break
			else:
				inattr = False
			node = path[-1]
			if isinstance(node, xsc.Element):
				return (True, xsc.enterattrs, xsc.entercontent)
			if inattr and isinstance(node, xsc.Text):
				return (True, )
			else:
				return (xsc.entercontent, )

	class filtertopdownattrwithoutcontent(xsc.WalkFilter):
		def filternode(self, node):
			if isinstance(node, xsc.Element):
				return (True, xsc.entercontent, xsc.enterattrs)
			elif isinstance(node, (xsc.Attr, xsc.Text)):
				return (True, )
			else:
				return (xsc.entercontent, )

	yield check, node, filtertopdown, ["div", "div.tr", "div.tr.th", "div.tr.td"]
	yield check, node, filterbottomup, ["div.tr.th", "div.tr.td", "div.tr", "div"]
	yield check, node, filtertopdownattrs, ["div", "div.class.i", "div.tr", "div.tr.id.b", "div.tr.th", "div.tr.td"]
	yield check, node, filterbottomupattrs, ["div.class.i", "div.tr.id.b", "div.tr.th", "div.tr.td", "div.tr", "div"]
	yield check, node, filtertopdowntextonlyinattr, ["div", "div.class.i", "div.class.i.#", "div.tr", "div.tr.id.b", "div.tr.id.b.#", "div.tr.th", "div.tr.td"]
	yield check, node, filtertopdownattrwithoutcontent, ["div", "div.tr", "div.tr.th", "div.tr.th.#", "div.tr.td", "div.tr.td.#", "div.tr.id", "div.class"]


def test_walkgetitem():
	e = html.div(
		1,
		html.div(
			2,
			html.div(
				3
			)
		)
	)
	isdiv = xsc.FindTypeAll(html.div)
	assert str(e.walknode(isdiv)[0]) == "123"
	assert str(e.walknode(isdiv)[-1]) == "3"
	py.test.raises(IndexError, e.walknode(isdiv).__getitem__, 3)
	py.test.raises(IndexError, e.walknode(isdiv).__getitem__, -4)
	assert str(e.walkpath(isdiv)[0][-1]) == "123"
	assert str(e.walkpath(isdiv)[-1][-1]) == "3"
	py.test.raises(IndexError, e.walkpath(isdiv).__getitem__, 3)
	py.test.raises(IndexError, e.walkpath(isdiv).__getitem__, -4)



#! /usr/bin/env/python
# -*- coding: utf-8 -*-

## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
## Copyright 1999-2008 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/__init__.py for the license


from ll.xist import xsc
from ll.xist.ns import html, php


def test_pretty():
	def check(node, result):
		assert node.pretty().bytes() == result

	tests = [
		(html.p("apple", "tree"), "<p>appletree</p>"),
		(html.p("apple", html.br(), "tree"), "<p>apple<br />tree</p>"),
		(html.p(php.php("apple")), "<p>\n\t<?php apple?>\n</p>"),
		(html.p(php.php("apple"), "tree"), "<p><?php apple?>tree</p>"),
		(
			html.div(2*html.p("apple", "tree"), html.br()),
			"<div>\n\t<p>appletree</p>\n\t<p>appletree</p>\n\t<br />\n</div>"
		),
		(
			html.div(
				php.php("apple"),
				html.p("apple", "tree"),
				html.div(
					html.p("apple"),
					html.p("tree"),
				),
				html.br()
			),
			"<div>\n\t<?php apple?>\n\t<p>appletree</p>\n\t<div>\n\t\t<p>apple</p>\n\t\t<p>tree</p>\n\t</div>\n\t<br />\n</div>"
		),
	]
	for (got, exp) in tests:
		yield check, got, exp
